/*  DESCRIPTION: */
/* 	DOOM graphics stuff for Atari/TT Xbios. */
/*  */

#include <signal.h>

#include "atari.h"

#include "i_system.h"
#include "v_video.h"

#include "doomdef.h"
#include "r_local.h"
#include "am_map.h"

static int	VideoDone=0;	/* Video initialized ? */

int fbnum=0;	/* Screen buffer to draw to */

int		pixel_size;
int		bpp;
lighttable_t	truecolor_palette[256];
extern byte	*orig_colormaps;

extern	unsigned long fuzzmask;	/*  for fuzz effect in 15/16 bit mode */

unsigned short	atari_oldvmode;
void		*atari_oldvbase;
unsigned long	atari_screen[2];	/* game screen */
void		*atari_ecralloc[2];
unsigned long	atari_couleur0;

unsigned long	atari_offset;	/* offset for game screen in real screen */

void I_AllocateScreensAtStart(void)
{
	int width,height;
	int i;

	I_ScanCookies();

	/* Allocate DMA buffer */

	atari_dmabuffer=Mxalloc(512*4*2,0);
	memset(atari_dmabuffer,0,512*4*2);

	/* --- Read video properties --- */

	width=SCREENWIDTH;
	height=SCREENHEIGHT;
	switch (cookie_vdo)
	{
		case VDO_TT:
			height=240;  	/* 480 8bits <-> 240 16 bits */
			atari_offset=SCREENWIDTH*40*2;
			break;
		case VDO_F30:
			switch (Montype())
			{
				case 0:		/* MONO */
					I_Error("Heretic does not work on a monochrome monitor\n");
					break;
				case 1:		/* RVB : 320x200 */
				case 3:		/* TV : 320x200 */
					atari_offset=0;
					break;
				case 2:		/* VGA : 320x240 */
					atari_offset=SCREENWIDTH*20*2;
					height=240;
					break;
			}
			break;
	}

	/* --- Allocate screens */

	for (i=0;i<2;i++)
	{
		atari_ecralloc[i]=Mxalloc((width*height*2)+256,0);

		if (atari_ecralloc[i]==NULL)
			I_Error("Not enough memory for screens.\n");

		atari_screen[i]=(unsigned long )(atari_ecralloc[i]+16UL) & 0xFFFFFFF0UL;
	
		memset((byte *)atari_screen[i],0,width*height*2);
        }
}

void I_ShutdownGraphics(void)
{
	void	*oldpile;

	if (VideoDone==-1)
	{
		Supexec(I_ShutdownInterrupts);

		switch(cookie_vdo)
		{
			case VDO_TT:
				Setscreen(-1,atari_oldvbase,-1,-1);
				EsetShift(atari_oldvmode);
				break;
			case VDO_F30:
				/* Bordure noire */

				oldpile=(void *)Super(0);
				VIDEL_COLOR0=atari_couleur0;
				Super(oldpile);

				Setscreen(-1,atari_oldvbase,-1,-1);
			        Vsetmode(atari_oldvmode);
				break;
		}
		VideoDone=0;
	}

	Mfree(atari_ecralloc[0]);
	Mfree(atari_ecralloc[1]);
}

/*  */
/*  I_StartTic */
/*  */
short cur_mouseb=0;
unsigned char cur_clavier[128];
short cur_joystick=0;
unsigned long cur_joypad=0;

void I_StartTic (void)
{
	event_t event;

	if (atari_CtrlCPressed!=0)
		I_Quit();

	/* Scan keyboard changes */

#define KEY_DOWN(a) (atari_clavier[a])
#define KEY_PRESS(a) (atari_clavier[a] && !cur_clavier[a])
#define KEY_WASPRESSED(a) (cur_clavier[a] && !atari_clavier[a])
#define PROCESS_KEY(k,t)					\
	if (KEY_PRESS(k))					\
	{							\
			event.type = ev_keydown;		\
			event.data1 = t;			\
			D_PostEvent(&event);			\
	}							\
	if (KEY_WASPRESSED(k))					\
	{							\
			event.type = ev_keyup;			\
			event.data1 = t;			\
			D_PostEvent(&event);			\
	}

	PROCESS_KEY(SCANCODE_CURSORBLOCKRIGHT, KEY_RIGHTARROW);
	PROCESS_KEY(SCANCODE_CURSORBLOCKLEFT, KEY_LEFTARROW);
	PROCESS_KEY(SCANCODE_CURSORBLOCKDOWN, KEY_DOWNARROW);
	PROCESS_KEY(SCANCODE_CURSORBLOCKUP, KEY_UPARROW);

	PROCESS_KEY(SCANCODE_ESCAPE, KEY_ESCAPE);
	PROCESS_KEY(SCANCODE_ENTER, KEY_ENTER);
	PROCESS_KEY(SCANCODE_KEYPADENTER, KEY_ENTER);
	PROCESS_KEY(SCANCODE_TAB, KEY_TAB);
	PROCESS_KEY(SCANCODE_SPACE, ' ');

	PROCESS_KEY(SCANCODE_F1, KEY_F1);
	PROCESS_KEY(SCANCODE_F2, KEY_F2);
	PROCESS_KEY(SCANCODE_F3, KEY_F3);
	PROCESS_KEY(SCANCODE_F4, KEY_F4);
	PROCESS_KEY(SCANCODE_F5, KEY_F5);
	PROCESS_KEY(SCANCODE_F6, KEY_F6);
	PROCESS_KEY(SCANCODE_F7, KEY_F7);
	PROCESS_KEY(SCANCODE_F8, KEY_F8);
	PROCESS_KEY(SCANCODE_F9, KEY_F9);
	PROCESS_KEY(SCANCODE_F10, KEY_F10);
	PROCESS_KEY(SCANCODE_F11, KEY_F11);
	PROCESS_KEY(SCANCODE_F12, KEY_F12);
		
	PROCESS_KEY(SCANCODE_1, '1');
	PROCESS_KEY(SCANCODE_2, '2');
	PROCESS_KEY(SCANCODE_3, '3');
	PROCESS_KEY(SCANCODE_4, '4');
	PROCESS_KEY(SCANCODE_5, '5');
	PROCESS_KEY(SCANCODE_6, '6');
	PROCESS_KEY(SCANCODE_7, '7');
	PROCESS_KEY(SCANCODE_8, '8');
	PROCESS_KEY(SCANCODE_9, '9');
	PROCESS_KEY(SCANCODE_0, '0');

	PROCESS_KEY(SCANCODE_MINUS, KEY_MINUS);
	PROCESS_KEY(SCANCODE_KEYPADMINUS, KEY_MINUS);
	PROCESS_KEY(SCANCODE_EQUAL, KEY_EQUALS);
	PROCESS_KEY(SCANCODE_KEYPADPLUS, KEY_EQUALS);

	PROCESS_KEY(SCANCODE_BACKSPACE, KEY_BACKSPACE);
	PROCESS_KEY(SCANCODE_BREAK, KEY_PAUSE);

	PROCESS_KEY(SCANCODE_LEFTSHIFT, KEY_RSHIFT);
	PROCESS_KEY(SCANCODE_RIGHTSHIFT, KEY_RSHIFT);
	PROCESS_KEY(SCANCODE_LEFTCONTROL, KEY_RCTRL);
	PROCESS_KEY(SCANCODE_LEFTALT, KEY_RALT);

	PROCESS_KEY(SCANCODE_Q, 'q');
	PROCESS_KEY(SCANCODE_W, 'w');
	PROCESS_KEY(SCANCODE_E, 'e');
	PROCESS_KEY(SCANCODE_R, 'r');
	PROCESS_KEY(SCANCODE_T, 't');
	PROCESS_KEY(SCANCODE_Y, 'y');
	PROCESS_KEY(SCANCODE_U, 'u');
	PROCESS_KEY(SCANCODE_I, 'i');
	PROCESS_KEY(SCANCODE_O, 'o');
	PROCESS_KEY(SCANCODE_P, 'p');

	PROCESS_KEY(SCANCODE_A, 'a');
	PROCESS_KEY(SCANCODE_S, 's');
	PROCESS_KEY(SCANCODE_D, 'd');
	PROCESS_KEY(SCANCODE_F, 'f');
	PROCESS_KEY(SCANCODE_G, 'g');
	PROCESS_KEY(SCANCODE_H, 'h');
	PROCESS_KEY(SCANCODE_J, 'j');
	PROCESS_KEY(SCANCODE_K, 'k');
	PROCESS_KEY(SCANCODE_L, 'l');

	PROCESS_KEY(SCANCODE_Z, 'z');
	PROCESS_KEY(SCANCODE_X, 'x');
	PROCESS_KEY(SCANCODE_C, 'c');
	PROCESS_KEY(SCANCODE_V, 'v');
	PROCESS_KEY(SCANCODE_B, 'b');
	PROCESS_KEY(SCANCODE_N, 'n');
	PROCESS_KEY(SCANCODE_M, 'm');

	PROCESS_KEY(SCANCODE_INSERT, KEY_INSERT);
	PROCESS_KEY(SCANCODE_HOME, KEY_HOME);
	PROCESS_KEY(SCANCODE_PAGEUP, KEY_PAGEUP);
	PROCESS_KEY(SCANCODE_REMOVE, KEY_DELETE);
	PROCESS_KEY(SCANCODE_END, KEY_END);
	PROCESS_KEY(SCANCODE_PAGEDOWN, KEY_PAGEDOWN);

	PROCESS_KEY(SCANCODE_KEYPADDIVIDE, '/');
	PROCESS_KEY(SCANCODE_KEYPADMULTIPLY, '*');

	memcpy(cur_clavier,atari_clavier,128);

	/* Scan mouse changes */

	if ((atari_mouseb != cur_mouseb) || atari_mousex || atari_mousey)
	{
		event.type = ev_mouse;
		event.data1 = atari_mouseb;
		event.data2 = atari_mousex << 2;
		event.data3 = atari_mousey << 2;

		D_PostEvent(&event);		

		cur_mouseb=atari_mouseb;
		atari_mousex=atari_mousey=0;
	}

#define JOY_DOWN(a) (atari_joystick & a)
#define JOY_PRESS(a) ( (atari_joystick & a) && !(cur_joystick & a) )
#define JOY_WASPRESSED(a) ( (cur_joystick & a) && !(atari_joystick & a) )
#define PROCESS_JOY(k,t)		 \
	if (JOY_PRESS(k))		\
	{				\
		event.type = ev_keydown;\
		event.data1 = t;	\
		D_PostEvent(&event);	\
	}				\
	if (JOY_WASPRESSED(k))		\
	{				\
		event.type = ev_keyup;	\
		event.data1 = t;	\
		D_PostEvent(&event);	\
	}

	if (atari_joystick != cur_joystick)
	{
		PROCESS_JOY(JS_UP,KEY_UPARROW);
		PROCESS_JOY(JS_DOWN,KEY_DOWNARROW);
		PROCESS_JOY(JS_LEFT,KEY_LEFTARROW);
		PROCESS_JOY(JS_RIGHT,KEY_RIGHTARROW);
		PROCESS_JOY(JS_FIRE,KEY_RCTRL);

		cur_joystick=atari_joystick;
	}

	/* Scan joypad 0 changes */
	/* Simulate keyboard */

#define PAD_DOWN(a) (atari_joypad0 & a)
#define PAD_PRESS(a) ( (atari_joypad0 & a) && !(cur_joypad & a) )
#define PAD_WASPRESSED(a) ( (cur_joypad & a) && !(atari_joypad0 & a) )
#define PROCESS_PAD(k,t)					\
	if (PAD_PRESS(k))					\
	{							\
			event.type = ev_keydown;		\
			event.data1 = t;			\
			D_PostEvent(&event);			\
	}							\
	if (PAD_WASPRESSED(k))					\
	{							\
			event.type = ev_keyup;			\
			event.data1 = t;			\
			D_PostEvent(&event);			\
	}

	if (atari_joypad0 != cur_joypad)
	{
		PROCESS_PAD(JP_UP,KEY_UPARROW);
		PROCESS_PAD(JP_DOWN,KEY_DOWNARROW);
		PROCESS_PAD(JP_LEFT,KEY_LEFTARROW);
		PROCESS_PAD(JP_RIGHT,KEY_RIGHTARROW);

		PROCESS_PAD(JP_OPTION,KEY_ESCAPE);
		PROCESS_PAD(JP_PAUSE,KEY_PAUSE);

		PROCESS_PAD(JP_FIRE0,KEY_RALT);
		PROCESS_PAD(JP_FIRE1,KEY_RCTRL);
		PROCESS_PAD(JP_FIRE2,' ');		

		PROCESS_PAD(JP_K1,'1');
		PROCESS_PAD(JP_K2,'2');
		PROCESS_PAD(JP_K3,'3');
		PROCESS_PAD(JP_K5,'4');
		PROCESS_PAD(JP_KASTER,'5');
		PROCESS_PAD(JP_K0,'6');
		PROCESS_PAD(JP_KNUM,'7');

		PROCESS_PAD(JP_KL,KEY_RALT);
		PROCESS_PAD(JP_KL,KEY_LEFTARROW);
		PROCESS_PAD(JP_KR,KEY_RALT);
		PROCESS_PAD(JP_KR,KEY_RIGHTARROW);

		PROCESS_PAD(JP_KX,KEY_RSHIFT);
		PROCESS_PAD(JP_KY,KEY_ENTER);
		PROCESS_PAD(JP_KZ,KEY_TAB);

		cur_joypad=atari_joypad0;
	}
}


/*  */
/*  I_FinishUpdate */
/*  */
extern patch_t *PatchINumbers[10];

void I_Update (void)
{
   static int	lasttic;
    int		tics;
    int		i;
	int		c,d,u;

    /* draws little dots on the bottom of the screen */
    if (devparm)
    {

	i = I_GetTime();
	tics = i - lasttic;
	lasttic = i;

	if (tics>999) tics=999;
	c=tics / 100;
	d=(tics-c*100) / 10;
	u=(tics-c*100-d*10);

	if (PatchINumbers[c])
	{
		V_DrawPatch(320-2-24,0,PatchINumbers[c]);
		V_DrawPatch(320-2-16,0,PatchINumbers[d]);
		V_DrawPatch(320-2-8,0,PatchINumbers[u]);
	}
    }

	/* Permutation des ecrans */

	if (pixel_size==1)
	{
		convert_c2p(screen,(byte *)(atari_screen[fbnum]+atari_offset),320,200,(cookie_vdo==VDO_TT));
	}

	Setscreen(-1,atari_screen[fbnum],-1,-1);
	fbnum ^= 1;				

	if (pixel_size!=1)
	{
		screen=(byte *)(atari_screen[fbnum]+atari_offset);
		R_InitBuffer(scaledviewwidth,viewheight); /* mettre a jour ylookup */
	}
}


/*  */
/*  I_ReadScreen */
/*  */
void I_ReadScreen (byte* scr)
{
    memcpy (scr, screen, SCREENWIDTH*SCREENHEIGHT*pixel_size);
}

/*  */
/*  I_SetPalette */
/*  */
void I_SetPalette(byte *palette)
{
	int		i;
	byte		*palette1=palette;
	unsigned long	tc=0;
	unsigned short	TT_palette[256];
	unsigned long	F30_palette[256];

	switch(pixel_size)
	{
		case 1:
			palette1=palette;
			switch(cookie_vdo)
			{
				case VDO_TT:
					for (i=0;i<256;i++)
					{
						int r,v,b;

						r = gammatable[usegamma][*palette++];	
						v = gammatable[usegamma][*palette++];	
						b = gammatable[usegamma][*palette++];	
					
						tc=((r>>4)<<8)|((v>>4)<<4)|(b>>4);
						TT_palette[i]=tc;
					}
					EsetPalette(0,256,&TT_palette[0]);
					break;
				case VDO_F30:
					for(i = 0; i < 256; i++)
					{
						int r,v,b;

						r = gammatable[usegamma][*palette++];	
						v = gammatable[usegamma][*palette++];
						b = gammatable[usegamma][*palette++];

						F30_palette[i]=(r<<16)|(v<<8)|b;
					}
					VsetRGB(0,256,&F30_palette[0]);
					break;
			}
			/* Remplir les colormaps avec les valeurs des octets */
			for (i=0;i<256;i++)
			{
				int y;

				for (y=0;y<NUMCOLORMAPS+2;y++)
				{
					unsigned long c;

					c=colormaps[y*256+i] & 255;

					c|=(c<<24)|(c<<16)|(c<<8);

					colormaps[y*256+i]=c;
				}
			}
			break;
		case 2:
		case 3:
			for(i = 0; i < 256; i++)
			{
				/* Palettes de lumiere */

				int		r1,v1,b1;
				int		c,y,r,v,b;

				c=orig_colormaps[i]*3;

				r1=gammatable[usegamma][palette[c]];
				v1=gammatable[usegamma][palette[c+1]];
				b1=gammatable[usegamma][palette[c+2]];	

				for(y=0;y<NUMCOLORMAPS;y++)
				{
					r=r1-(y*r1)/(NUMCOLORMAPS-1);
					v=v1-(y*v1)/(NUMCOLORMAPS-1);
					b=b1-(y*b1)/(NUMCOLORMAPS-1);	
					switch (bpp)
					{
						case 15:
							tc = ((r>>3)<<10)|((v>>3)<<5)|(b>>3);
							tc |= tc<<16;
							break;
						case 16:
							tc = ((r>>3)<<11)|((v>>2)<<5)|(b>>3);
							tc |= tc<<16;
							break;
						case 24:
							tc = (r<<16)|(v<<8)|b;
							break;
					}
					colormaps[y*256+i]=tc;			
				}

				/* Invulnerabilite */
	
				colormaps[NUMCOLORMAPS*256+i]=colormaps[orig_colormaps[32*256+i]];

				/* Noir */
	
				colormaps[(NUMCOLORMAPS+1)*256+i]=0;

				/* Palettes normales */

				r=gammatable[usegamma][*palette1++];
				v=gammatable[usegamma][*palette1++];
				b=gammatable[usegamma][*palette1++];

				switch (bpp)
				{
					case 15:
						tc = ((r>>3)<<10)|((v>>3)<<5)|(b>>3);
						tc |= tc<<16;
						break;
					case 16:
						tc = ((r>>3)<<11)|((v>>2)<<5)|(b>>3);
						tc |= tc<<16;
						break;
					case 24:
						tc = (r<<16)|(v<<8)|b;
						break;
				}
				truecolor_palette[i]=tc;
			}
			break;
	}
}

void I_InitGraphics(void)
{
	short new_mode;
	void *oldpile;
	int i;

	fbnum=0;

	init_c2p();

	switch(cookie_vdo)
	{
		case VDO_TT:
			atari_oldvbase=Logbase();
			atari_oldvmode=EgetShift();

			Setscreen(-1,atari_screen[1],-1,-1);
			EsetShift(TT_LOW);

			bpp=8;

			break;
		case VDO_F30:
			atari_oldvmode=Vsetmode(-1);
			atari_oldvbase=Logbase();	

			new_mode=atari_oldvmode & ( VGA + PAL );
			if (video8)
			{
				new_mode|=BPS8;
				atari_offset>>=1; /* diviser par 2 */
				bpp=8;
			} else {
				new_mode|=BPS16;
				bpp=16;
				dblbuffer=true;
			}
			if (Montype()==2)
				new_mode|=VERTFLAG;

			Setscreen(-1,atari_screen[1],-1,-1);
			Vsetmode(new_mode);

			/* Bordure noire */

			oldpile=(void *)Super(0);
			atari_couleur0=VIDEL_COLOR0;
			VIDEL_COLOR0=NULL;
			Super(oldpile);

			break;
		default:
			I_Error("You need a TT or a Falcon video chip\n");
	}

	/* Interruptions */

	Supexec(I_InitInterrupts);

	/* OK */

	VideoDone=-1;

	bpp=16;
	switch (bpp)
	{
		case 8:
			R_DrawColumn=R_DrawColumn8;
			R_DrawFuzzColumn=R_DrawFuzzColumn8;
			R_DrawTranslatedColumn=R_DrawTranslatedColumn8;
			R_DrawSpan=R_DrawSpan8;
			R_DrawSkyColumn=R_DrawSkyColumn8;

			R_DrawColumnLow=R_DrawColumnLow8;
			R_DrawFuzzColumnLow=R_DrawFuzzColumnLow8;
			R_DrawTranslatedColumnLow=R_DrawTranslatedColumnLow8;
			R_DrawSpanLow=R_DrawSpanLow8;
			R_DrawSkyColumnLow=R_DrawSkyColumnLow8;

			AM_drawFline=AM_drawFline8;
			V_DrawPatch=V_DrawPatch8;
			V_DrawShadowedPatch=V_DrawShadowedPatch8;
			V_DrawFuzzPatch=V_DrawFuzzPatch8;
			pixel_size=1;
			break;
		case 15:
			R_DrawColumn=R_DrawColumn16;
			R_DrawFuzzColumn=R_DrawFuzzColumn16;
			R_DrawTranslatedColumn=R_DrawTranslatedColumn16;
			R_DrawSpan=R_DrawSpan16;
			R_DrawSkyColumn=R_DrawSkyColumn16;

			R_DrawColumnLow=R_DrawColumnLow16;
			R_DrawFuzzColumnLow=R_DrawFuzzColumnLow16;
			R_DrawTranslatedColumnLow=R_DrawTranslatedColumnLow16;
			R_DrawSpanLow=R_DrawSpanLow16;
			R_DrawSkyColumnLow=R_DrawSkyColumnLow16;

			AM_drawFline=AM_drawFline16;
			V_DrawPatch=V_DrawShadowedPatch=V_DrawPatch16;
			V_DrawFuzzPatch=V_DrawFuzzPatch16;
			pixel_size=2;
			fuzzmask=0x3DEF3DEF;
			break;
		case 16:
			R_DrawColumn=R_DrawColumn16;
			R_DrawFuzzColumn=R_DrawFuzzColumn16;
			R_DrawTranslatedColumn=R_DrawTranslatedColumn16;
			R_DrawSpan=R_DrawSpan16;
			R_DrawSkyColumn=R_DrawSkyColumn16;

			R_DrawColumnLow=R_DrawColumnLow16;
			R_DrawFuzzColumnLow=R_DrawFuzzColumnLow16;
			R_DrawTranslatedColumnLow=R_DrawTranslatedColumnLow16;
			R_DrawSpanLow=R_DrawSpanLow16;
			R_DrawSkyColumnLow=R_DrawSkyColumnLow16;

			AM_drawFline=AM_drawFline16;
			V_DrawPatch=V_DrawShadowedPatch=V_DrawPatch16;
			V_DrawFuzzPatch=V_DrawFuzzPatch16;
			pixel_size=2;
			fuzzmask=0x7BEF7BEF;
			break;
		case 24:
			R_DrawColumn=R_DrawColumn24;
			R_DrawFuzzColumn=R_DrawFuzzColumn24;
			R_DrawTranslatedColumn=R_DrawTranslatedColumn24;
			R_DrawSpan=R_DrawSpan24;
			R_DrawSkyColumn=R_DrawSkyColumn24;

			R_DrawColumnLow=R_DrawColumnLow24;
			R_DrawFuzzColumnLow=R_DrawFuzzColumnLow24;
			R_DrawTranslatedColumnLow=R_DrawTranslatedColumnLow24;
			R_DrawSpanLow=R_DrawSpanLow24;
			R_DrawSkyColumnLow=R_DrawSkyColumnLow24;

			AM_drawFline=AM_drawFline24;
			V_DrawPatch=V_DrawShadowedPatch=V_DrawPatch24;
			V_DrawFuzzPatch=V_DrawFuzzPatch24;
			pixel_size=3;
			break;
	}

	I_SetPalette(W_CacheLumpName("PLAYPAL",PU_CACHE));

	signal(SIGINT, (void (*)(int)) I_Quit);

	memset(cur_clavier,0,128);

	screen = (byte *)(atari_screen[0]+atari_offset);
}
